/*

***

simple biofeedback device code.
KansasFest 2014 hackfest entry
author: Charles Mangin <option8@option8.com>
***


the original one-off piece was bodged together, but i've tried my best to recreate the connections in the included Fritzing file (biofeedback.fzz)

First, connect the game port to the breadboard. you'll need pin 1 for power, pin 8 for ground and pin 6 to read the joystick input.

Connect pin6 of the game port to a wiper output of the potentiometer chip (i'm using an AD5206 since my other project requires 4 pot outputs. you can use a simpler chip, as long as you can generate ~150k resistance to match the Apple II joystick pots.

connect power and ground to the arduino.

one arduino input is used for the "reset" button. connect the other pole of the button to ground.

an analog input on the arduino is used for the actual galvanic sensor. connect it to one of the sensor pads, and the other side of the pad to ground (pulldown).

the other pad is connected to 5V. when a person bridges these, the variable voltage is read by the arduino, and assigned a number. 

based on the "trend" of that voltage input up or down, the potentiometers are written (in this case via SPI) to a value the Apple II can read.





*/

#include <SPI.h>

// set pin A0 as the slave select for the digital pot:
const int slaveSelectPin = 10;
const int TrimPin = A5;
const int Butt0Pin = 2;

  int channelX = 5; // which pot pin are we writing to?
  int InputVoltage; // reading from finger pads
  int InputVoltageSeed; // initial reading on reset
  float RunningAverage;
  float Trend; // running average, normalized to 1 == steady state

int joyX = 64; // initial joystick state = centered

void setup()
{
  // set the slaveSelectPin as an output:
  pinMode (slaveSelectPin, OUTPUT);
  pinMode (Butt0Pin, INPUT_PULLUP);
  

  // initialize SPI:
  SPI.begin(); 

  Serial.begin(9600); // for debugging

}

void loop()
{
      InputVoltage = analogRead(TrimPin); // read input with human in loop
      
      /*
      pin/button reads initial Val
      
      running average of value:
      RA = .9 * RA + .1 * Val
      
      write pin 0-255 based on trend up/down from initial value.
      */
      
      if (!digitalRead(Butt0Pin)) {
        InputVoltageSeed = InputVoltage;
        RunningAverage = InputVoltageSeed;
        Serial.println("RESET");

      }
      
      RunningAverage = .666 * RunningAverage + .334 * InputVoltage;
      
      Trend = RunningAverage/InputVoltageSeed; // flat = 1. trending up >1, trending down <1
          
  
      // trend < 1, joyX = 0 = calmer, green
      // trend > 1, joyX = 128 = anxious, red
      // trend == 1, joyX = 64 = medium, blue
      
  
  
      if(Trend < .9) {
        joyX = 0;
      } else if(Trend > 1.1) {
        joyX = 128;
      } else if(Trend < 1) { 
        joyX = 43;
      } else { 
        joyX = 85;
      }
  
      //int joyX = 128 * Trend ; // reads 0 - 1023. 1023/4 = 255. 2 pots to write to == 0-128 vals.
  
  
      Serial.print(InputVoltage);
      
      Serial.print(" -- ");
      
      Serial.print(RunningAverage);
      
      Serial.print(" -- ");
      
      Serial.print(Trend);
      
      Serial.print(" -- ");
      
      Serial.println(joyX);
      
      delay(100);

    // write the values to the POTs
      digitalPotWrite(channelX, joyX);
      digitalPotWrite(channelX - 1, joyX);// 100k pots, 2nd pot required to reach 150k.



}


void digitalPotWrite(int address, int value) {
  // take the SS pin low to select the chip:
  digitalWrite(slaveSelectPin,LOW);
  //  send in the address and value via SPI:
  SPI.transfer(address);
  SPI.transfer(value);
  // take the SS pin high to de-select the chip:
  digitalWrite(slaveSelectPin,HIGH); 
}


